/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Geometry.cpp,v 1.1 2003/01/26 10:41:08 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#include <PalmOS.h>
#include "gfx/Geometry.h"


void Geometry::uniteBounds(RectangleType *rect1Bound, RectangleType *rect2Bound, RectangleType *resultBound)
{
	// If one of the rectangles has zero area, then use the dimensions of the
	// other one as the result.
	if ((rect1Bound->extent.x == 0) || (rect1Bound->extent.y == 0))
	{
		resultBound->topLeft.x = rect2Bound->topLeft.x;	
		resultBound->topLeft.y = rect2Bound->topLeft.y;	
		resultBound->extent.x = rect2Bound->extent.x;	
		resultBound->extent.y = rect2Bound->extent.y;	

		return;
	}

	if ((rect2Bound->extent.x == 0) || (rect2Bound->extent.y == 0))
	{
		resultBound->topLeft.x = rect1Bound->topLeft.x;	
		resultBound->topLeft.y = rect1Bound->topLeft.y;	
		resultBound->extent.x = rect1Bound->extent.x;	
		resultBound->extent.y = rect1Bound->extent.y;	

		return;
	}


	// Do real calculations
	Coord rect1RightEdge = rect1Bound->topLeft.x + rect1Bound->extent.x;
	Coord rect2RightEdge = rect2Bound->topLeft.x + rect2Bound->extent.x;

	if (rect1Bound->topLeft.x < rect2Bound->topLeft.x)
	{
		resultBound->topLeft.x = rect1Bound->topLeft.x;
	}
	else
	{
		resultBound->topLeft.x = rect2Bound->topLeft.x;
	}

	if (rect1RightEdge > rect2RightEdge)
	{
		resultBound->extent.x = rect1RightEdge - resultBound->topLeft.x;
	}
	else
	{
		resultBound->extent.x = rect2RightEdge - resultBound->topLeft.x;
	}

	Coord rect1BottomEdge = rect1Bound->topLeft.y + rect1Bound->extent.y;
	Coord rect2BottomEdge = rect2Bound->topLeft.y + rect2Bound->extent.y;

	if (rect1Bound->topLeft.y < rect2Bound->topLeft.y)
	{
		resultBound->topLeft.y = rect1Bound->topLeft.y;
	}
	else
	{
		resultBound->topLeft.y = rect2Bound->topLeft.y;
	}

	if (rect1BottomEdge > rect2BottomEdge)
	{
		resultBound->extent.y = rect1BottomEdge - resultBound->topLeft.y;
	}
	else
	{
		resultBound->extent.y = rect2BottomEdge - resultBound->topLeft.y;
	}
}
