/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Device.cpp,v 1.4 2003/05/30 12:16:44 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#include <PalmOS.h>
#include "Device.h"
#include "../../Rsc/RazorRsc.h"    // Needed for ROM incompatible alert

UInt32  Device::romVersion;
Boolean Device::supports30;
Boolean Device::supports31;
Boolean Device::supports35;
Boolean Device::supports40;
Boolean Device::supports50;

Boolean Device::isArmed;

UInt32 Device::creatorID;
UInt16 Device::appVersionNum;


void Device::panic(const Char *message)
{
    FrmCustomAlert(DevicePanicAlert, message, NULL, NULL);
    ErrThrow(0L);
}


void Device::init()
{
	UInt16 cardNo;
	LocalID dbID;
	SysCurAppDatabase(&cardNo, &dbID);
	DmDatabaseInfo(cardNo, dbID, NULL, NULL, &Device::appVersionNum, NULL, NULL, NULL, NULL, NULL, NULL, NULL, &Device::creatorID);

	// Determine version of Palm OS
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &Device::romVersion);
	Device::supports30 = (Device::romVersion >= Device::RomVersion30);
	Device::supports31 = (Device::romVersion >= Device::RomVersion31);
	Device::supports35 = (Device::romVersion >= Device::RomVersion35);
	Device::supports40 = (Device::romVersion >= Device::RomVersion40);
	Device::supports50 = (Device::romVersion >= Device::RomVersion50);

	// Determine CPU type	
	UInt32 cpuID;
    FtrGet(sysFtrCreator, sysFtrNumProcessorID, &cpuID);

#ifdef USE_SDK50
	Device::isArmed = (cpuID >= sysFtrNumProcessorARM720T && cpuID <= sysFtrNumProcessorARM710A);
#else
	Device::isArmed = false;
#endif
}

void Device::release()
{
}


Err Device::romVersionCompatible(UInt32 minVersion, UInt32 maxVersion, UInt16 launchFlags)
{
	// See if the device contains a suitable OS ROM.
	if ((Device::romVersion < minVersion) || (Device::romVersion > maxVersion))
	{
		if ((launchFlags & (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
			(sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
		{
			FrmAlert(RomIncompatibleAlert);
		
			// Palm OS 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (sysGetROMVerMajor(Device::romVersion) == 1)
			{
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
			}
		}
		
		return sysErrRomIncompatible;
	}

	return errNone;
}



Boolean Device::supportsTrap(UInt16 trapNum)
{
	void* procP = SysGetTrapAddress(trapNum);
	if ((procP != NULL) && (procP != SysGetTrapAddress(sysTrapSysUnimplemented)))
	{
		return true;
	}
	else
	{
		return false;
	}
}
