/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: Color.cpp,v 1.2 2003/05/29 15:45:25 teacy Exp $
 *
 * Copyright (c) 2003 Tilo Christ. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/

#include "./HostControl.h"
#include <PalmOS.h>
#include "gfx/Color.h"
#include "gfx/Display.h"
#include "Device.h"


Color::Color(UInt8 r, UInt8 g, UInt8 b)
{
	init(r, g, b);
}



void Color::init(UInt8 r, UInt8 g, UInt8 b)
{
	rgbColor.r = r;
	rgbColor.g = g;
	rgbColor.b = b;
	isIndexed = false;
}



IndexedColorType Color::getIndexedColor() const
{
	if (!isIndexed)
	{
		if (Device::supports35)
		{
			// Work around bug on Visor Prism. WinRGBToIndex will crash if current draw window is offscreen.
			const WinHandle drawWindow = WinGetDrawWindow();
			WinSetDrawWindow(WinGetDisplayWindow());
			rgbColor.index = WinRGBToIndex(&rgbColor);
			WinSetDrawWindow(drawWindow);
		}
		else
		{
			// Pre-OS 3.5 means grayscale
			UInt16 avgLum = (rgbColor.r + rgbColor.g + rgbColor.b) / 3;
			rgbColor.index = (avgLum >> (8 - Display::depth)) - 1;
		}
		
		isIndexed = true;				
	}

	return rgbColor.index;
}


RGBColorType Color::getRGBColor() const
{
	return rgbColor;
}


const Color Color::BLACK = Color(0,0,0);
const Color Color::WHITE = Color(255,255,255);
