/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: HardKeyManager.cpp,v 1.2 2003/02/15 11:55:46 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#include "Customization.h"
#include "input/HardKeyManager.h"

using namespace prvrazor;

void HardKeyManager::init()
{
	// Get the key repeat rate for when we want to restore it after the game.
	KeyRates(false, &oldInitDelay, &oldPeriod, &oldDoubleTapDelay, &oldQueueAhead);
		
	setMask(keysAllowedMask);
}


void HardKeyManager::destroy() 
{
	releaseHardKeys();
}

	
void HardKeyManager::captureHardKeys()
{
	// Set the keys we poll to not generate events.  This saves CPU cycles.
	KeySetMask(	~mask );
}
	

void HardKeyManager::releaseHardKeys()
{
	KeySetMask(keyBitsAll);
	KeyRates(true, &oldInitDelay, &oldPeriod, &oldDoubleTapDelay, &oldQueueAhead);
}
		

UInt32 HardKeyManager::getCurrentState()
{
#ifndef MODE_PROFILING
	return KeyCurrentState();
#else
	return 0;  // Always fake no keypresses during profiling.
#endif
}
	

void HardKeyManager::setMask(UInt32 mask)
{
	HardKeyManager::mask = mask;
}

	
UInt32 HardKeyManager::mask;

UInt16 HardKeyManager::oldInitDelay;
UInt16 HardKeyManager::oldPeriod;
UInt16 HardKeyManager::oldDoubleTapDelay;
Boolean HardKeyManager::oldQueueAhead;
