/*********************************************************************************
 *
 * Razor! Engine - A modular C++ presentation engine
 *
 * $Id: PalmOSDisplay.h,v 1.3 2003/05/29 15:46:33 teacy Exp $
 *
 * Copyright (c) 2000-2003 Tilo Christ. All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the  "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the Software 
 * is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all 
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 * INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE 
 * FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 **********************************************************************************/


#ifndef PALMOS_DISPLAY_H
#define PALMOS_DISPLAY_H

#include "Customization.h"
#include "RazorSections.h"
#include <PalmOS.h>
#include "gfx/Display.h"
#ifndef MODE_PURITY
#include "gfx/DirectDoubleBuffer.h"
#endif

namespace prvrazor
{


/**
 * PalmOSDisplay is the basic canvas that supports no special features in devices from PalmOS licensees.
 * It should work on all Palm OS devices. 
 */
class PalmOSDisplay : public Display
{
	public:

	PalmOSDisplay() SEC_RAZOR_INIT;
	
	virtual ~PalmOSDisplay() SEC_RAZOR_INIT;

	void beginDrawViewport();
		
	void endDrawViewport(RectangleType *bounds);

	void showViewport();

	void captureDisplay() SEC_RAZOR_INIT;
	void releaseDisplay() SEC_RAZOR_INIT;

	void initDisplay(const DisplayProperties& displayProperties) SEC_RAZOR_INIT;

	protected:
	void createViewportCanvas(const ViewportProperties& viewport) SEC_RAZOR_INIT;
	void resetDisplay() SEC_RAZOR_INIT;
		
#ifndef MODE_PURITY
	DirectDoubleBuffer *directDoubleBuffer;
#endif

	private:
	void saveDrawState(DrawStateType& drawState) SEC_RAZOR_INIT;
	void restoreDrawState(const DrawStateType& drawState) SEC_RAZOR_INIT;
    void initPhysicalDisplay() SEC_RAZOR_INIT;

	// DrawState save/restore
	Boolean displayCaptured;
	DrawStateType originalDrawState;
	DrawStateType presentationDrawState;
};

}

#endif
